using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace XnaRagdoll
{
    public interface ICamera
    {
    }

    public class Camera : Microsoft.Xna.Framework.GameComponent, ICamera
    {
        private GraphicsDeviceManager graphics;

        private Matrix projection;
        private Matrix view;
        private Vector3[] viewPara;
        private Matrix[] viewTrans;

        public Camera(Game game)
            : base(game)
        {
            graphics = (GraphicsDeviceManager)Game.Services.GetService(typeof(IGraphicsDeviceManager));

            viewPara = new Vector3[3];
            viewTrans = new Matrix[3];
        }

        public override void Initialize()
        {
            base.Initialize();
            InitializeCamera();
        }

        public void InitializeCamera()
        {
            viewPara[0] = new Vector3(0.0f, 50.0f, 250.0f);
            viewPara[1] = new Vector3(0.0f, 50.0f, 0.0f);
            viewPara[2] = Vector3.Up;
            for (int i = 0; i < 3; ++i)
                viewTrans[i] = Matrix.Identity;

            float aspectRatio = (float)graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, aspectRatio, 1.0f, 1.0e4f);
            CalcViewMatrix();
        }

        public Matrix View
        {
            get { return view; }
        }

        public Matrix Projection
        {
            get { return projection; }
        }

        private void CalcViewMatrix()
        {
            Vector3[] tmp = new Vector3[3];
            for (int i = 0; i < 3; ++i)
                tmp[i] = Vector3.Transform(viewPara[i], viewTrans[i]);
            view = Matrix.CreateLookAt(tmp[0], tmp[1], tmp[2]);
        }

        public Vector4 GetEyePosition()
        {
            Vector4 v = new Vector4(view.M11, view.M12, view.M13, 1.0f);
            return Vector4.Transform(v, viewTrans[0]);
        }

        #region operator
        public void Translate(Vector3 trans)
        {
            Matrix tmp = Matrix.CreateTranslation(trans);
            viewTrans[0] = tmp * viewTrans[0];
            viewTrans[1] = tmp * viewTrans[1];

            CalcViewMatrix();
        }

        public void Rotate(Matrix rot)
        {
            for (int i = 0; i < 3; ++i)
                viewTrans[i] = rot * viewTrans[i];
            CalcViewMatrix();
        }

        public void Zoom(float dist)
        {
            viewPara[0].Z += dist;
            if (viewPara[0].Z < 0.01f)
                viewPara[0].Z = 0.01f;
            CalcViewMatrix();
        }
        #endregion
    }
}
