float4x4 world : WORLD;
float4x4 viewProj : VIEWPROJECTION;
float4x4 worldViewProj : WORLDVIEWPROJECTION;

float4 globalLightPos : GLOBALLIGHTPOSITION;
float4 localLightPos : LOCALLIGHTPOSITION;
float4 localEyePos : LOCALEYEPOSITION;

float4 ambiColor = float4(0.7f, 0.4f, 0.4f, 1.0f);;
float4 diffColor = float4(0.7f, 0.4f, 0.4f, 1.0f);;

const float4 lightColor = float4(0.7f, 0.7f, 0.7f, 1.0f);
const float specExpon = 1.0f;


struct VS_INPUT
{
  float4 pos : POSITION;
  float3 normal : NORMAL;
};

struct PLANESHADOW_VS_OUTPUT
{
  float4 pos : POSITION;
  float4 color : COLOR0;
};

PLANESHADOW_VS_OUTPUT PlaneShadowVS(VS_INPUT In)
{
  PLANESHADOW_VS_OUTPUT Out = (PLANESHADOW_VS_OUTPUT)0;

  float4 shadowPos = mul(In.pos, world);
  shadowPos.xz += (shadowPos.xz - globalLightPos.xz) / (globalLightPos.y - shadowPos.y) * shadowPos.y;
  shadowPos.y = 0;
  Out.pos = mul(shadowPos, viewProj);
  Out.color = float4(0.0f, 0.0f, 0.0f, 1.0f);
  return Out;
}

float4 PlaneShadowPS(PLANESHADOW_VS_OUTPUT In) : COLOR0
{
  return In.color;
}

technique PlaneShadowShader
{
  pass P0
  {
    AlphaBlendEnable = true;
    SrcBlend = SrcAlpha;
    DestBlend = InvSrcAlpha;
    VertexShader = compile vs_2_0 PlaneShadowVS();
    PixelShader = compile ps_2_0 PlaneShadowPS();
  }
}

struct PHONG_VS_OUTPUT
{
  float4 pos : POSITION;
  float3 normal : TEXCOORD0;
  float3 view : TEXCOORD1;
  float3 light : TEXCOORD2;
};

PHONG_VS_OUTPUT PhongVS(VS_INPUT In)
{
  PHONG_VS_OUTPUT Out = (PHONG_VS_OUTPUT)0;
 
  Out.pos = mul(In.pos, worldViewProj);
  Out.view = normalize(localEyePos.xyz - In.pos.xyz);
  Out.light = normalize(localLightPos.xyz - In.pos.xyz);
  Out.normal = In.normal;

  return Out;
}
float4 PhongPS(PHONG_VS_OUTPUT In) : COLOR0
{
  float3 halfv = normalize(In.view + In.light);
  float4 litv = lit(dot(In.normal, In.light), dot(In.normal, halfv), specExpon);
  
  float4 diffContrib = diffColor * (0.5 * litv.y * lightColor + ambiColor);
  float4 specContrib = litv.y * litv.z * lightColor;
  return diffContrib + specContrib;
}
technique PhongShader
{
  pass P0
  {
    CullMode = CCW;
    AlphaBlendEnable = true;
    SrcBlend = SrcAlpha;
    DestBlend = InvSrcAlpha;
    
    VertexShader = compile vs_1_1 PhongVS();
    PixelShader = compile ps_2_0 PhongPS();
  }
}